/***********************************************************
 * $Id: fancydeflatechannel.h 1480 2015-02-11 17:56:06Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Christoph Lechleitner/IBCL <lech@ibcl.at>
 *
 * Header of BufferChannel
 * Channels reading/writing from/to in-RAM buffers with base64 encoding/decoding on write operations
 *
 ************************************************************/

#ifndef FANCY_ZIPCHANNEL_H_
#define FANCY_ZIPCHANNEL_H_

#include <fancychannel.h>

namespace Fancy {

class ZipEntry;

/**
 * A Channel that creates zip files.
 *
 * This implementation is thread-safe in the sense, that separate threads may
 * call read() and write() and yet-another thread my call close() or shutdown().
 * Concurrent read() or write() operations from two threads are not supported
 * and will most likely lead to unpredictable results.
 *
 * Please note, that you have to call close() or flush() to really finish the
 * generated output stream.
 *
 * After calling flush(), a new deflated stream may be started by writing more
 * data to this channel.
 */
class FANCYBASE_API ZipChannel : public virtual Channel {

public:
    typedef ObjPtr<ZipChannel> ptr_t;

    /**
     * Returns a ZipChannel instance.
     *
     *comment, is a comment for the entire zip file
     */
    static ptr_t newInstance(const Fancy::ObjPtr<Fancy::Channel> &channel,const std::string &comment);

    /**
     * Writes a Fancy::ZipEntry to the Channel
     */
    virtual void putNextEntry(const Fancy::ObjPtr<Fancy::ZipEntry> &entry) = 0;

protected:
    ZipChannel();
    ~ZipChannel();
};

}

#endif /* FANCY_DEFLATECHANNEL_H_ */
