/***********************************************************
 * $Id: fancydeflatechannel.h 1480 2015-02-11 17:56:06Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Christoph Lechleitner/IBCL <lech@ibcl.at>
 *
 * Header of BufferChannel
 * Channels reading/writing from/to in-RAM buffers with base64 encoding/decoding on write operations
 *
 ************************************************************/

#ifndef FANCY_ZIPENTRY_H_
#define FANCY_ZIPENTRY_H_

#include <fancyobject.h>

namespace Fancy {

class UtcTimestamp;

template<class T> class ArrayObject;


/**
 * An Object representing on entry in a zip file.
 *
 * Use this obeject to write data with Fancy::ZipChannel
 */
class FANCYBASE_API ZipEntry : public virtual Object {

public:
    typedef ObjPtr<ZipEntry> ptr_t;

    /**
     * Creates a new Instance of ZipEntry, with the name as specifed
     */
    static ptr_t newInstance(const std::string &name);

    /**
     * Returns the name of the entry.
     */
    virtual std::string getName() = 0;

    /**
     * Returns the comment string for the entry, or empty string if none.
     */
    virtual std::string getComment() = 0;

    /**
     * Sets the optional comment string for the entry.
     */
    virtual void setComment(const std::string &comment) = 0;

    /**
     * Returns the size of the compressed entry data, or -1 if not known.
     */
    virtual uint64_t getCompressedSize() = 0;

    /**
     * Sets the size of the compressed entry data
     */
    virtual void setCompressedSize(uint64_t csize) = 0;

    /**
     * Returns the CRC-32 checksum of the uncompressed entry data, or -1 if not known.
     */
    virtual uint64_t getCrc() = 0;

    /**
     * Sets the CRC-32 checksum of the uncompressed entry data.
     */
    virtual void setCrc(uint64_t crc) = 0;

    /**
     * Returns the extra field data for the entry, or 0 if none.
     */
    virtual Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > getExtra() = 0;

    /**
     * Sets the optional extra field data for the entry.
     */
    virtual void setExtra(const Fancy::ObjPtr<Fancy::ArrayObject<uint8_t> > &extra) = 0;

    /**
     * Returns the compression method of the entry, or -1 if not specified.
     */
    virtual uint32_t getMethod() = 0;

    /**
     * Sets the compression method for the entry.
     */
    virtual void setMethod(uint32_t method) = 0;

    /**
     * Returns the uncompressed size of the entry data, or -1 if not known.
     */
    virtual uint64_t getSize() = 0;

    /**
     * Sets the uncompressed size of the entry data.
     */
    virtual void setSize(uint64_t size) = 0;

    /**
     * Returns the modification time of the entry, or 0 if not specified.
     */
    virtual Fancy::ObjPtr<Fancy::UtcTimestamp> getTime() = 0;

    /**
     * Sets the modification time of the entry.
     */
    virtual void setTime(const Fancy::ObjPtr<Fancy::UtcTimestamp> &time)= 0 ;

    /**
     * Returns true if this is a directory entry.
     */
    virtual bool isDirectory() = 0;

protected:
    ZipEntry();
    ~ZipEntry();
};

}

#endif /* FANCY_ZIPENTRY_H_ */
